@@ -1,7 +1,9 @@
 use std::borrow::Borrow;
 use std::fmt::Debug;
 use std::hash::{BuildHasher, Hash, Hasher};
+use std::iter::FromIterator;
 use std::mem::MaybeUninit;
+use std::ops::Index;
 
 use crate::pair::Pair;
 
@@ -414,6 +416,7 @@ impl<'a, K, V> Iter<'a, K, V> {
     }
 }
 
+#[allow(clippy::borrow_as_ptr)]
 impl<'a, K, V> Iterator for Iter<'a, K, V> {
     type Item = &'a Pair<K, V>;
 
@@ -486,8 +489,94 @@ impl<'a, K: Debug, V: Debug, H: BuildHasher, A: Allocator> Debug for HashMap<K,
     }
 }
 
+impl<'a, K: Eq + Hash, V, Q: Eq + Hash + 'a, H: BuildHasher, A: Allocator> Index<&'a Q>
+    for HashMap<K, V, H, A>
+where
+    K: Borrow<Q>,
+{
+    type Output = V;
+    fn index(&self, idx: &'a Q) -> &V {
+        self.get(idx).unwrap()
+    }
+}
+
+impl<K: Eq + Hash, V: PartialEq, H: BuildHasher, A: Allocator> PartialEq for HashMap<K, V, H, A> {
+    fn eq(&self, other: &Self) -> bool {
+        if core::ptr::eq(self, other) {
+            return true;
+        }
+        for Pair(k, v) in self {
+            if other.get(k).map_or(false, |v2| v.eq(v2)) {
+                return false;
+            }
+        }
+        for Pair(k, v) in other {
+            if self.get(k).map_or(false, |v2| v.eq(v2)) {
+                return false;
+            }
+        }
+
+        true
+    }
+}
+
+impl<K: Eq + Hash, V: Eq, H: BuildHasher, A: Allocator> Eq for HashMap<K, V, H, A> {}
+
+impl<K: Eq + Hash, V, H: BuildHasher + Default, A: Allocator + Default> FromIterator<Pair<K, V>>
+    for HashMap<K, V, H, A>
+{
+    fn from_iter<I>(it: I) -> Self
+    where
+        I: IntoIterator<Item = Pair<K, V>>,
+    {
+        let mut ret = Self::new();
+        for Pair(k, v) in it {
+            ret.insert(k, v);
+        }
+        ret
+    }
+}
+
+impl<K: Eq + Hash, V, H: BuildHasher, A: Allocator> Extend<Pair<K, V>> for HashMap<K, V, H, A> {
+    fn extend<I>(&mut self, it: I)
+    where
+        I: IntoIterator<Item = Pair<K, V>>,
+    {
+        for Pair(k, v) in it {
+            self.insert(k, v);
+        }
+    }
+}
+
+impl<K: Eq + Hash, V, H: BuildHasher + Default, A: Allocator + Default> FromIterator<(K, V)>
+    for HashMap<K, V, H, A>
+{
+    fn from_iter<I>(it: I) -> Self
+    where
+        I: IntoIterator<Item = (K, V)>,
+    {
+        let mut ret = Self::new();
+        for (k, v) in it {
+            ret.insert(k, v);
+        }
+        ret
+    }
+}
+
+impl<K: Eq + Hash, V, H: BuildHasher, A: Allocator> Extend<(K, V)> for HashMap<K, V, H, A> {
+    fn extend<I>(&mut self, it: I)
+    where
+        I: IntoIterator<Item = (K, V)>,
+    {
+        for (k, v) in it {
+            self.insert(k, v);
+        }
+    }
+}
+
 ///
 /// A [`HashSet`] that has a stable ABI and obtains storage backed by an [`Allocator`]
+#[allow(clippy::derive_hash_xor_eq)]
 #[derive(
     Clone,
     Debug, /* may not be good to derive(Debug) here. Maybe should provide impl instead */
@@ -497,8 +586,17 @@ pub struct HashSet<K, H: BuildHasher = BuildHasherDefault<XLangHasher>, A: Alloc
     inner: HashMap<K, (), H, A>,
 }
 
+impl<K: Eq + Hash, H: BuildHasher, A: Allocator> PartialEq for HashSet<K, H, A> {
+    fn eq(&self, other: &Self) -> bool {
+        self.inner == other.inner
+    }
+}
+
+impl<K: Eq + Hash, H: BuildHasher, A: Allocator> Eq for HashSet<K, H, A> {}
+
 impl<K, H: BuildHasher + Default, A: Allocator + Default> HashSet<K, H, A> {
     /// Constructs a new [`HashSet`] with a [`Default`] allocator and [`Default`] hasher
+    #[must_use]
     pub fn new() -> Self {
         Self {
             inner: HashMap::new(),
@@ -506,8 +604,15 @@ impl<K, H: BuildHasher + Default, A: Allocator + Default> HashSet<K, H, A> {
     }
 }
 
+impl<K, H: BuildHasher + Default, A: Allocator + Default> Default for HashSet<K, H, A> {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl<K, H: BuildHasher, A: Allocator + Default> HashSet<K, H, A> {
     /// Constructs a new [`HashSet`] with the given hasher and a [`Default`] allocator
+    #[must_use]
     pub fn with_hasher(hasher: H) -> Self {
         Self {
             inner: HashMap::with_hasher(hasher),
@@ -517,6 +622,7 @@ impl<K, H: BuildHasher, A: Allocator + Default> HashSet<K, H, A> {
 
 impl<K, H: BuildHasher + Default, A: Allocator> HashSet<K, H, A> {
     /// Constructs a new [`HashSet`] with the given allocator and a [`Default`] hasher
+    #[must_use]
     pub fn new_in(alloc: A) -> Self {
         Self {
             inner: HashMap::new_in(alloc),
@@ -526,6 +632,7 @@ impl<K, H: BuildHasher + Default, A: Allocator> HashSet<K, H, A> {
 
 impl<K, H: BuildHasher, A: Allocator> HashSet<K, H, A> {
     /// Constructs a new [`HashSet`] with the given hasher and allocator
+    #[must_use]
     pub fn with_hasher_in(hasher: H, alloc: A) -> Self {
         Self {
             inner: HashMap::with_hasher_in(hasher, alloc),
@@ -550,6 +657,7 @@ impl<K: Eq + Hash, H: BuildHasher, A: Allocator> HashSet<K, H, A> {
 
     ///
     /// Inserts `val` into the set if it is not already present (according to the [`Eq`] implementation), or returns it (as an Err) instead.
+    #[allow(clippy::missing_errors_doc)] // Not an Error to return the `K`
     pub fn insert(&mut self, val: K) -> Result<(), K> {
         if self.inner.get(&val).is_some() {
             Err(val)
@@ -559,6 +667,42 @@ impl<K: Eq + Hash, H: BuildHasher, A: Allocator> HashSet<K, H, A> {
         }
     }
 }
+impl<K: Eq + Hash, H: BuildHasher + Default, A: Allocator + Default> FromIterator<K>
+    for HashSet<K, H, A>
+{
+    fn from_iter<I>(iter: I) -> Self
+    where
+        I: IntoIterator<Item = K>,
+    {
+        let mut out = Self::new();
+        for item in iter {
+            drop(out.insert(item));
+        }
+        out
+    }
+}
+
+impl<K: Eq + Hash, H: BuildHasher + Default, A: Allocator + Default> Extend<K>
+    for HashSet<K, H, A>
+{
+    fn extend<I>(&mut self, iter: I)
+    where
+        I: IntoIterator<Item = K>,
+    {
+        for item in iter {
+            drop(self.insert(item));
+        }
+    }
+}
+
+/// Creates a [`HashSet`] from given elements, discarding duplicates
+#[macro_export]
+macro_rules! set{
+    {$($expr:expr),*} => {{
+        let __array = [$($expr),*];
+        $crate::collection::HashSet::from_iter(::core::iter::IntoIterator::into_iter(__array))
+    }}
+}
 
 /// An [`Iterator`] over the values in a [`HashSet`]
 pub struct SetIter<'a, K>(Keys<'a, K, ()>);
